% For use with Matlab
% Copyright (C) 2020 -- Jerome Mendes <jermendes@gmail.com> or <jermendes@isr.uc.pt>, Rui Araujo <rui@isr.uc.pt>
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza. "Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control". Applied Sicences, 2020.
% The "uSelf-FLC toolbox" comes with ABSOLUTELY NO WARRANTY;
% In case of publication of any application of this method, please, cite the work:
% Jerome Mendes, Ricardo Maia, Rui Araujo, and  Francisco A.A. Souza.
% Self-Evolving Fuzzy Controller Composed of Univariate Fuzzy Control Rules.
% Applied Sciences, 2020, 10(17), 5836.
% DOI: https://doi.org/10.3390/app10175836
% http://www.isr.uc.pt/~jermendes/    ;     http://www.isr.uc.pt/~rui/

function [x] = Initialization( universe_x, Rules_ini, eta_aux, Min_command_signal )
    %% Description
    % This function initializes the struct x, i.e. the initial fuzzy control parameters
    % (Algorithm 2 Step 2)
    %% Function inputs
    % universe_x - Minimum and maximum values of the inputs variables
    % Rules_ini -  Number of initial membership functions (rules) per input variable
    % eta - Threshold \eta_j
    % Min_command_signal -  minimum admissible control value
    %% Function output
    % x - Struct with all fuzzy control parameters

%% Variables Definition
n_var = size(universe_x, 1); % number of inputs variables

%% Initialization of the struct x - Algorithm 1 Steps 3-8 (Section 3.2)
for j = 1:n_var % for all input variables
    
    %% Save Variables Limits
    x(j).Limit = [universe_x(j,1) universe_x(j,2)]; % Definition of universe of discourses  of input variable j
    
    %% Save Number of initial membership functions (rules) per input variable
    x(j).Rules_ini = Rules_ini(j); %Definiton of the number of initial fuzzy rules for input variable j
    
    %% Threshold \eta_j
    x(j).eta =  (x(j).Limit(1,2) - x(j).Limit(1,1) ) / eta_aux(j); % minimal distance beetween MFs, for input variable j
    
    %% Antecedents Design  - Algorithm 2 Step 2.a
    x(j).Rules = zeros( Rules_ini(j), 4); % Initialize all values with zeros
    
    for i = 1:x(j).Rules_ini % for all rules of input variable j
        
        if i == 1 % First rule
            
            range = (x(j).Limit(1,2) - x(j).Limit(1,1) ) / ( x(j).Rules_ini - 1 );
            
            x(j).Rules(i,2) = x(j).Limit(1,1); % parameter b_{j,i}
            
            x(j).Rules(i,1) = x(j).Limit(1,1); % parameter a_{j,i}
            
            x(j).Rules(i,3) = x(j).Limit(1,1) + range;
            
            
        elseif i == x(j).Rules_ini % Last rule
            
            x(j).Rules(i,2) = x(j).Limit(1,2); % parameter b_{j,i}
            
            x(j).Rules(i,1) = x(j).Limit(1,2) - range; % parameter a_{j,i}
            
            x(j).Rules(i,3) = x(j).Rules(i,2); % parameter c_{j,i}
            
        else % Others rule
            
            x(j).Rules(i,2) = x(j).Rules(i-1,2) + range; % parameter b_{j,i}
            
            x(j).Rules(i,1) = x(j).Rules(i-1,2); % parameter a_{j,i}
            
            x(j).Rules(i,3) =  x(j).Rules(i,2) + range; % parameter c_{j,i}
            
        end
        
        x(j).Rules(i,4) = Min_command_signal; % consequent parameter
        
        x(j).Rules(i,5) = 0; % N parameter
        
        x(j).Rules(i,6) = 1; % Sample time created
        
        x(j).Rules(i,7) = 0; % min N parameter
        
    end
end

end
